/* eslint-disable consistent-return */
import { TextableChannel } from 'eris';
import { Client } from '.';
import PageCommand from '../commands/page';

export default class PBX {
  public client: Client;

  constructor(client: Client) {
    this.client = client;

    this.install();
    this.pageDTMF();
  }

  protected install() {
    this.client.util.ari.start('page-dtmf');
  }

  public pageDTMF() {
    this.client.util.ari.on('StasisStart', async (event, channel) => {
      if (event.application !== 'page-dtmf') return;
      const message = await (<TextableChannel> this.client.guilds.get(this.client.config.guildID).channels.get('501089664040697858')).getMessage('775604192013320203');
      if (!message) return channel.hangup();
      const member = await this.client.db.Staff.findOne({ extension: channel.caller.number }).lean().exec();
      if (!member) return channel.hangup();
      const pager = await this.client.db.PagerNumber.findOne({ individualAssignID: member.userID }).lean().exec();
      if (!pager) return channel.hangup();
      let status = 0;
      const pagerNumber: string[] = [];
      const pagerCode: string[] = [];
      channel.answer();
      await channel.play({
        media: 'sound:please-enter-the-pn',
      }, undefined);
      channel.on('ChannelDtmfReceived', async (ev) => {
        if (status === 0) {
          if (ev.digit === '#') {
            await channel.play({
              media: 'sound:please-enter-the-pc',
            }, undefined);
            status = 1;
            return null;
          }
          pagerNumber.push(ev.digit);
        } else if (status === 1) {
          if (ev.digit === '#') {
            await channel.play({
              media: 'sound:pls-hold-process-tx',
            }, undefined);

            const Page = <PageCommand> this.client.commands.get('page');
            const page = await Page.page(pagerNumber.join(''), pager.num, pagerCode.join(''), message);
            if (page.status === true) {
              const playback = await channel.play({
                media: 'sound:page-delivered',
              }, undefined);
              playback.on('PlaybackFinished', () => channel.hangup());
            } else if (page.status === false) {
              try {
                const ch = await this.client.getDMChannel(member.userID);
                if (ch) {
                  ch.createMessage(`***An error has occurred while trying to process your page request over the PBX.***\n*${page.message}*\n\nPlease check your parameters and try again.`);
                }
              } catch {
                this.client.util.handleError(new Error(page.message));
              }
              const playback = await channel.play({
                media: 'sound:request-error',
              }, undefined);
              playback.on('PlaybackFinished', () => channel.hangup());
            }
            return null;
          }
          pagerCode.push(ev.digit);
        }
      });
    });
  }
}
